#Task 4: DER parsing

types_verbose = \
{
    0x02: 'INTEGER',
    0x03: 'BIT STRING',
    0x04: 'OCTET STRING',
    0x05: 'NULL',
    0x06: 'OBJECT',
    0x13: 'PRINTABLESTRING',
    0x14: 'T61STRING',
    0x17: 'UTCTIME',
    0x30: 'SEQUENCE',
    0x31: 'SET'
}

def get_tag(byte_array, offset):
    '''returns tag and offset'''
    tag = byte_array[offset]
    if tag in types_verbose:
        return types_verbose[tag], offset + 1
    else:
        return tag, offset + 1

# length is defined by byte or bytes on given offset - return tuple (length, new_offset)
def get_length(byte_array, offset):
    '''returns length and new offset'''
    l = byte_array[offset]
    offset += 1
    assert(l != 0x80), 'indefinite length'
    if l < 128:
        return l, offset
    else:
        l -= 128
        return int.from_bytes(byte_array[offset: offset + l], byteorder='big', signed=False), offset + l

def get_value(byte_array, offset,  length):
    '''returns value and offset'''
    return byte_array[offset: offset + length], offset + length

def tlv(byte_array, offset=0, depth=0):
    offset_backup = offset  # position where tlv starts
    t, offset = get_tag(byte_array, offset)
    l, offset = get_length(byte_array, offset)

    if (t in ['SEQUENCE', 'SET']) or not(t in types_verbose.values()):
        # print('\t' * depth, f'offset={offset_backup}, tag={t}, length={l}')
        sequence_end = offset + l
        while (offset < sequence_end):
            offset = tlv(byte_array, offset, depth + 1)
            # '00', '30', '81', '89', '02', '81', '81
            # '00', '30', '82', '01', '09', '02', '82', '01', '01
    elif  t == 'BIT STRING':
        offset = offset + 1
    else:
        v, offset = get_value(byte_array, offset, l)
        if t == 'INTEGER':
            print(int.from_bytes(v, byteorder='big'))
        # print('\t' * depth, f'offset={offset_backup}, tag={t}, length={l} value={integers}')
    return offset


with open('pub_12345_2048.der', 'rb') as der_file:
    byte_array = der_file.read()
    tlv(byte_array)
